from notecoin.okex.v5.client import (AccountClient, AssetClient, MarketClient,
                                     PublicClient, TradeClient)
from notetool.secret import read_secret

api_key = read_secret(cate1='coin', cate2='okex', cate3='api_key')
secret_key = read_secret(cate1='coin', cate2='okex', cate3='secret_key')
passphrase = read_secret(cate1='coin', cate2='okex', cate3='passphrase')

account = AccountClient(api_key, secret_key, passphrase)
fundingAPI = AssetClient(api_key, secret_key, passphrase)
market = MarketClient(api_key, secret_key, passphrase)
publicAPI = PublicClient(api_key, secret_key, passphrase)
tradeAPI = TradeClient(api_key, secret_key, passphrase)


class OkexCoin:
    def __init__(self, coin_id='METIS-USDT', count=0.):
        self.coin_id = coin_id
        self.count = count
        self.price_in = self.current_price
        self.price = self.price_in

    @property
    def current_price(self):
        self.price = float(market.get_ticker(self.coin_id).data[0]['last'])
        return self.price

    @property
    def money(self):
        return self.current_price * self.count

    def buy(self):
        print("sell")
        return tradeAPI.place_order(instId=self.coin_id, tdMode='cash', side='buy', ordType='market', sz='50')

    def sell(self):
        coin_dict = dict([(line['ccy'], line['availBal']) for line in account.get_account().data[0]['details']])
        count = coin_dict[self.coin_id.split('-')[0]]
        return tradeAPI.place_order(instId=self.coin_id, tdMode='cash', side='sell', ordType='market', sz=count)

    @staticmethod
    def instance_by_account(data):
        okex = OkexCoin(coin_id=f"{data['ccy']}-USDT", count=float(data['availBal']))
        return okex

    @staticmethod
    def instance_by_new(coin_id):
        okex = OkexCoin(coin_id=coin_id)
        okex.buy()
        return okex

    def watch(self):
        if self.money > 52 or self.money < 48:
            self.sell()

    def __str__(self):
        price = self.current_price
        return f"{self.coin_id}\t{self.count}\t{self.price_in}\t{price}\t{self.count * price}"

    def to_json(self):
        return {
            "coin_id": self.coin_id,
            # "count": self.count,
            # "price_in": self.price_in,
            # "price": price,
            "usdt": self.count * self.price
        }



from waitress import serve
from flask import Flask
from notecoin.okex.strategy.strategy1 import Strategy1

app = Flask(__name__)


@app.route('/')
def index():
    return 'success'


Strategy1.register(app, route_base='/')


#serve(app, host="0.0.0.0", port=8884)
app.run(host='0.0.0.0', port=8444)



from flask_classful import FlaskView
from notecoin.okex.strategy.domain import OkexCoin
from notecoin.okex.strategy.utils import account_api


class Strategy1(FlaskView):
    def __init__(self):
        self.usdt = 0
        self.coin_map = {}

    def load(self):
        data = account_api.get_account().data[0]
        self.coin_map = {}
        for detail in data['details']:
            if detail['ccy'] == 'USDT':
                self.usdt = detail['availBal']
                continue
            coin = OkexCoin.instance_by_account(detail)
            if coin.money > 1:
                self.coin_map[coin.coin_id] = coin

    def to_json(self):
        return {
            "res": self.usdt,
            "coins": [coin.to_json() for coin in self.coin_map.values()]
        }

    def update(self):
        self.load()
        for coin in self.coin_map.values():
            coin.watch()
        return self.to_json()



from .client import Client
from .consts import *


class IndexAPI(Client):

    def __init__(self, api_key, api_secret_key, passphrase, use_server_time=False):
        Client.__init__(self, api_key, api_secret_key, passphrase, use_server_time)

    # get index constituents
    def get_index_constituents(self, instrument_id):
        return self._request_without_params(GET, INDEX_GET_CONSTITUENTS + str(instrument_id) + '/constituents')



from notecoin.okex.v5.client.base import BaseClient
from notecoin.okex.v5.consts import GET


class PublicClient(BaseClient):

    def __init__(self, *args, **kwargs):
        super(PublicClient, self).__init__(*args, **kwargs)

    def get_instruments(self, instType, uly=None, instId=None):
        uri = '/api/v5/public/instruments'
        params = {'instType': instType, 'uly': uly, 'instId': instId}
        return self._request_with_params(GET, uri, params)

    def get_deliver_history(self, instType, uly, after=None, before=None, limit=None):
        uri = '/api/v5/public/delivery-exercise-history'
        params = {'instType': instType, 'uly': uly, 'after': after, 'before': before, 'limit': limit}
        return self._request_with_params(GET, uri, params)

    def get_open_interest(self, instType, uly=None, instId=None):
        uri = '/api/v5/public/open-interest'
        params = {'instType': instType, 'uly': uly, 'instId': instId}
        return self._request_with_params(GET, uri, params)

    def get_funding_rate(self, instId):
        uri = '/api/v5/public/funding-rate'
        params = {'instId': instId}
        return self._request_with_params(GET, uri, params)

    def funding_rate_history(self, instId, after=None, before=None, limit=None):
        uri = '/api/v5/public/funding-rate-history'
        params = {'instId': instId, 'after': after, 'before': before, 'limit': limit}
        return self._request_with_params(GET, uri, params)

    def get_price_limit(self, instId):
        uri = '/api/v5/public/price-limit'
        params = {'instId': instId}
        return self._request_with_params(GET, uri, params)

    def get_opt_summary(self, uly, expTime=None):
        uri = '/api/v5/public/opt-summary'
        params = {'uly': uly, 'expTime': expTime}
        return self._request_with_params(GET, uri, params)

    def get_estimated_price(self, instId):
        uri = '/api/v5/public/estimated-price'
        params = {'instId': instId}
        return self._request_with_params(GET, uri, params)

    def discount_interest_free_quota(self, ccy=None):
        uri = '/api/v5/public/discount-rate-interest-free-quota'
        params = {'ccy': ccy}
        return self._request_with_params(GET, uri, params)

    def get_system_time(self):
        uri = '/api/v5/public/time'
        return self._request_without_params(GET, uri)

    def get_liquidation_orders(self, instType, mgnMode=None, instId=None, ccy=None, uly=None,
                               alias=None, state=None, before=None, after=None, limit=None):
        uri = '/api/v5/public/liquidation-orders'
        params = {'instType': instType, 'mgnMode': mgnMode, 'instId': instId, 'ccy': ccy, 'uly': uly,
                  'alias': alias, 'state': state, 'before': before, 'after': after, 'limit': limit}
        return self._request_with_params(GET, uri, params)

    def get_mark_price(self, instType, uly=None, instId=None):
        uri = '/api/v5/public/mark-price'
        params = {'instType': instType, 'uly': uly, 'instId': instId}
        return self._request_with_params(GET, uri, params)

    def get_tier(self, instType, tdMode, uly=None, instId=None, ccy=None, tier=None):
        uri = '/api/v5/public/mark-price'
        params = {'instType': instType, 'tdMode': tdMode, 'uly': uly, 'instId': instId, 'ccy': ccy, 'tier': tier}
        return self._request_with_params(GET, uri, params)



from notecoin.okex.v5.client.base import BaseClient
from notecoin.okex.v5.consts import GET, POST


class SubAccountClient(BaseClient):
    def __init__(self, *args, **kwargs):
        super(SubAccountClient, self).__init__(*args, **kwargs)

    def balances(self, subAcct):
        uri = '/api/v5/account/subaccount/balances'
        params = {"subAcct": subAcct}
        return self._request_with_params(GET, uri, params)

    def bills(self, ccy=None, type=None, subAcct=None, after=None, before=None, limit=None):
        uri = '/api/v5/asset/subaccount/bills'
        params = {"ccy": ccy, 'type': type, 'subAcct': subAcct, 'after': after, 'before': before, 'limit': limit}
        return self._request_with_params(GET, uri, params)

    def delete(self, pwd, subAcct, apiKey):
        uri = '/api/v5/users/subaccount/delete-apikey'
        params = {'pwd': pwd, 'subAcct': subAcct, 'apiKey': apiKey}
        return self._request_with_params(POST, uri, params)

    def reset(self, pwd, subAcct, label, apiKey, perm, ip=None):
        uri = '/api/v5/users/subaccount/modify-apikey'
        params = {'pwd': pwd, 'subAcct': subAcct, 'label': label, 'apiKey': apiKey, 'perm': perm, 'ip': ip}
        return self._request_with_params(POST, uri, params)

    def create(self, pwd, subAcct, label, Passphrase, perm=None, ip=None):
        uri = '/api/v5/users/subaccount/apikey'
        params = {'pwd': pwd, 'subAcct': subAcct, 'label': label, 'Passphrase': Passphrase, 'perm': perm, 'ip': ip}
        return self._request_with_params(POST, uri, params)

    def view_list(self, enable=None, subAcct=None, after=None, before=None, limit=None):
        uri = '/api/v5/users/subaccount/list'
        params = {'enable': enable, 'subAcct': subAcct, 'after': after, 'before': before, 'limit': limit}
        return self._request_with_params(GET, uri, params)

    def control_transfer(self, ccy, amt, froms, to, fromSubAccount, toSubAccount):
        uri = '/api/v5/asset/subaccount/transfer'
        params = {'ccy': ccy, 'amt': amt, 'from': froms, 'to': to, 'fromSubAccount': fromSubAccount,
                  'toSubAccount': toSubAccount}
        return self._request_with_params(POST, uri, params)
